<?php

/**
 * Class WC_TA_Checkout_Vat
 *
 * Handle all the checkout related VAT stuff
 */
class WC_TA_Checkout_Vat {

	/**
	 * Setup the class
	 */
	public function setup() {

		// Update the taxes on the checkout page whenever the order review template part is refreshed
		add_action( 'woocommerce_checkout_update_order_review', array(
			$this, 'update_taxes_on_update_order_review',), 10, 1 );

		// Update the taxes in the checkout process when the checkout is processed
		add_action( 'woocommerce_checkout_process', array( $this, 'update_taxes_on_check_process' ), 10 );

		// Update the taxes when the line taxes are calculated in the admin
		add_filter( 'woocommerce_ajax_calc_line_taxes', array( $this, 'update_taxes_on_calc_line_taxes' ), 10, 3 );
		add_action( 'woocommerce_cart_calculate_fees', array( $this, 'custom_tax_for_shipping'), 10, 1 );

		add_filter( 'woocommerce_package_rates', array( $this, 'custom_shipping_rate'), 20, 2 );
		add_action('woocommerce_review_order_before_submit', array( $this, 'vertex_brand_logo_checkout' ) );

		/* PLUGINS-2530 */
		add_filter( 'woocommerce_checkout_fields', array( $this, 'checkout_fields_trigger_refresh'), 9999 );
	}

	public function checkout_fields_trigger_refresh( $fields ) {

		$fields['billing']['billing_email']['class'][] = 'update_totals_on_change';

		return $fields;
	} /* PLUGINS-2530 END */


	public function vertex_brand_logo_checkout() {
	
		$vertex_api_call = WC()->session->get( 'vertex_api_call' );
		if ('yes' == $vertex_api_call) { ?>
			<tr><td colspan="2">Powered by <img src=" <?php echo esc_url(plugins_url('logo/logo.png', __FILE__)); ?>" width="20%" height="50%"/></td></tr>
		<?php 
		}
	}


	public function custom_shipping_rate( $rates, $package ) {
		$shipping_methods = WC()->session->get( 'chosen_shipping_methods' );
		$vertex_ses = WC()->session->get( 'vertex_ses' );
		$shipping_method = $shipping_methods[0]; 
		//if ($shipping_method) {
			/*foreach ( $rates as $rate_key => $rate ) {
				$taxrate = $rate->cost * ( $vertex_ses / 100 );
				$rates[$rate_key]->cost = $rate->cost;
				WC()->session->set('old_shipping_amount', $taxrate);
				$taxes = array();
				foreach ($rates[$rate_key]->taxes as $key => $tax) {
					if ( $rates[$rate_key]->taxes[$key] > 0 ) {
						$taxes[$key] = 0;
					}
				}
				$rates[$rate_key]->taxes = $taxes;
			}*/
		//}
		return $rates;
	}


	public function custom_tax_for_shipping( $cart ) {
		if ( is_admin() && ! defined('DOING_AJAX') ) {
			return;
		}

		if (isset(WC()->session->get('shipping_for_package_0')['rates'])) {
				
			foreach ( WC()->session->get('shipping_for_package_0')['rates'] as $method_id => $rate ) {
				if ( WC()->session->get('chosen_shipping_methods')[0] == $method_id ) {
					$rate_label = $rate->label; // The shipping method label name
					$vertex_ses = WC()->session->get( 'vertex_ses' );
					//$old_shipping_amount = WC()->session->get( 'old_shipping_amount' );
					
					$rate_cost_tax = floatval($rate->cost);

					$taxrate = $rate_cost_tax * ( $vertex_ses / 100 );

					$srate = WC()->session->get( 'old_shipping_amount' );
					$tax_name = WC()->session->get( 'ship_tax_name' );
					$country_code = WC()->session->get( 'ship_country_code' );
					$product_class = WC()->session->get( 'ship_product_class' );
					$chosen_payment_method = WC()->session->get('chosen_payment_method');

					if ('cod' != $chosen_payment_method ) {

						if ( 'S' == $product_class ) {
							// Add the fee (tax third argument disabled: false)
							$cart->add_fee( __( $country_code . ' ' . strtoupper($tax_name) . ' SERVICE', 'woocommerce'), $taxrate, false );
						} 
					}
				}
			}
		}  
	}

	/**
	 * Update taxes in cart
	 *
	 * @param object $transaction
	 * @param String $country_code
	 */
	private function update_taxes_in_cart( $transaction, $country_code ) {

		// Don't modify cart if taxes aren't supported
		if ( isset( $transaction->tax_supported ) && false === $transaction->tax_supported ) {
			return;
		}

		/* PLUGINS-2530 */
		$taxClass = get_option('woocommerce_shipping_tax_class');

		if (!empty( $taxClass ) && 'inherit' == $taxClass ) {
			
			update_option('woocommerce_shipping_tax_class', 'standard');
		}

		// The transaction lines
		$transaction_lines = $transaction->transaction_lines;

		// Check if there's a valid buyer tax number
		$valid_tax_number = false;
		if ( isset( $transaction->buyer_tax_number_valid ) && true === $transaction->buyer_tax_number_valid && true === $transaction->tax_deducted ) {
			$valid_tax_number = true;
		}
		// Check
		if ( count( $transaction_lines ) > 0 ) {

			// Create tax manager object
			$tax_manager = new WC_TA_Tax_Manager();
			foreach ( $transaction_lines as $sl ) {

				$product_class = $sl->product_class;
				$srate = $sl->taxes;
				$tax_name = $srate[0]->tax_name;

				if ('S' == $product_class ) {

					WC()->session->set( 'ship_rate_value', $srate );
					WC()->session->set( 'ship_product_class', $product_class );
					WC()->session->set( 'ship_tax_name', $tax_name );
					WC()->session->set( 'ship_country_code', $country_code );
				} else {
					
					WC()->session->__unset( 'ship_product_class' );
				}	
			}

			$product_class = WC()->session->get( 'ship_product_class' );

			if (isset($product_class) && 'S' == $product_class) {
			
				$my_cnt = count($transaction_lines);
				unset($transaction_lines[$my_cnt-1]);
			}

			// Loop
			foreach ( $transaction_lines as $tl ) {

				// Fetch the cart item linked to the transaction line
				$cart_item = WC()->cart->get_cart_item( $tl->custom_id );

				$id = $cart_item['data']->is_type( 'variation' ) ? $cart_item['variation_id'] : $cart_item['product_id'];
				
				if ('yes' === WC_TA_Integration::$enable_v3) {
					// Add the new product class for this product
					$rate = $tl->taxes;
					WC()->session->set( 'vertex_ses', $rate[0]->rate );					
					$tax_manager->add_product_tax_class( $id, $rate[0]->tax_name );
					
					if (strtoupper($rate[0]->tax_name) == 'EUROPEAN VAT') {
						/* translators: %s: tax_name */
						$tax_manager->add_tax_rate( $rate[0]->tax_name, $rate[0]->rate, strtoupper( sprintf( __( '%1$s %2$s others', 'woocommerce-taxamo' ), $country_code, $rate[0]->tax_name ) ) );
					} else {
						/* translators: %s: tax_name */
						$tax_manager->add_tax_rate( $rate[0]->tax_name, $rate[0]->rate, strtoupper( sprintf( __( '%1$s %2$s', 'woocommerce-taxamo' ), $country_code, $rate[0]->tax_name ) ) );
					}
					
				} else {
					// Set all VAT to 0 if we've got a valid tax number because it's a B2B sale
					if ( true === $valid_tax_number ) {
						
						// Add the new tax rate for this transaction line
						$tax_manager->add_tax_rate( $tl->product_type, 0.00, __( 'European VAT', 'woocommerce-taxamo' ) );

						continue;
					}

					// Don't modify tax rates for default products
					if ( 'default' == $tl->product_type ) {
						continue;
					}

					$tax_manager->add_product_tax_class( $id, $tl->product_type );
					/* translators: %s: tax_name */
					$tax_manager->add_tax_rate( $tl->product_type, $tl->tax_rate, strtoupper( sprintf( __( 'VAT %1$s %2$s', 'woocommerce-taxamo' ), $country_code, $tl->product_type ) ) );
				}
			}
		}
	}

	/**
	 * Catch the update order review action and update taxes to selected billing country
	 *
	 * @param $post_data
	 */
	public function update_taxes_on_update_order_review( $post_data ) {

		// Parse the string
		parse_str( $post_data, $post_arr );
		global $woocommerce;
		$products = $woocommerce->cart->get_cart();
		$is_virtual = array();
		$has_physical = 'false';

		foreach ( $products as $product ) {

			$product_id = $product['product_id'];
			$is_virtual[] = get_post_meta( $product_id, '_virtual', true );	
		}
		if (in_array('no', $is_virtual)) {
			$has_physical = 'true';
		}

		if ('true' != $has_physical) {
			
			WC()->session->__unset( 'ship_product_class' );
			WC()->session->__unset( 'vertex_api_call' );
			return;
		} else {
			// The billing country
			if (array_key_exists('ship_to_different_address', $post_arr) && '1' == sanitize_text_field( $post_arr['ship_to_different_address'])) {

				$billing_country = sanitize_text_field( $post_arr['shipping_country'] );
			} else {
				$billing_country = sanitize_text_field( $post_arr['billing_country'] );
			}

			// Transaction extra's
			$transaction_extra = array();

			// Check for self declaration and enable_v1
			if ('yes' == WC_TA_Integration::$enable_v1 || 'yes' == WC_TA_Integration::$enable_v2) {
				// Set force country code
				$transaction_extra['force_country_code'] = $billing_country;
			}
			// Check for enable_v3
			if ('yes' === WC_TA_Integration::$enable_v3) {

				$transaction_extra['buyer_name'] = $post_arr['billing_first_name'] . ' ' . $post_arr['billing_last_name'];

				$transaction_extra['buyer_email'] = $post_arr['billing_email'];
				
				if (!array_key_exists('shipping_address_1', $post_arr) || '' == sanitize_text_field( $post_arr['shipping_address_1']) ) {

					$add_to_ship = array(
						'street_name' 	=> $post_arr['billing_address_1'] . ' ' . $post_arr['billing_address_2'],
						'city' 			=> $post_arr['billing_city'],
						'postal_code' 	=> $post_arr['billing_postcode']
					);

				} else {

					$add_to_ship = array(
						'street_name' 	=> $post_arr['shipping_address_1'] . ' ' . $post_arr['shipping_address_2'],
						'city' 			=> $post_arr['shipping_city'],
						'postal_code' 	=> $post_arr['shipping_postcode']
					);
				}

				$transaction_extra['ship_to_address'] = $add_to_ship;
			}
			// Check for VAT number
			if ( isset( $post_arr['vat_number'] ) && '' !== $post_arr['vat_number'] ) {

				// Set VAT number
				$transaction_extra['buyer_tax_number'] = str_replace( ' ', '', $post_arr['vat_number'] );
			}

			$chosen_payment_method = $post_arr['payment_method'];

			if ( 'cod' != $chosen_payment_method ) {


				// The cart manager
				$cart_manager = new WC_TA_Cart_Manager();

				// Setup request
				$request_calculate_tax = new WC_TA_Request_Calculate_Tax( $billing_country, $cart_manager->get_items_from_cart(), $transaction_extra );

				// Do request
				if ( $request_calculate_tax->do_request() ) {

					// Get the body
					$response_body = $request_calculate_tax->get_response_body();

					if ( isset( $response_body->transaction ) ) {
						// if enable v3
						if ('yes' === WC_TA_Integration::$enable_v3) {
						
							// Update the taxes in cart based on transaction lines
							$this->update_taxes_in_cart( $response_body->transaction, $response_body->transaction->tax_country_codes );
						} else {
							// Update the taxes in cart based on transaction lines
							$this->update_taxes_in_cart( $response_body->transaction, $response_body->transaction->tax_country_code );
						}
						WC()->session->set( 'vertex_api_call', 'yes' );	
					} else {
						WC()->session->__unset( 'ship_product_class' );
						WC()->session->__unset( 'vertex_api_call' );
					}

				} else {

					/**
					 * Check if AJAX is updated correctly is country doesn't match first try but does second try.
					 *
					 * @todo Better error handling.
					 * Block order button
					 */

					wc_add_notice( $request_calculate_tax->get_error_message(), 'error' );

				}
			} else {
				//wc_add_notice(esc_attr('Selected Payment Method not authorised to process further Tax Calculation. Please connect with administrator!', 'taxamo'), 'notice');
				WC()->session->__unset( 'ship_product_class' );
				WC()->session->__unset( 'vertex_api_call' );
			}
		}

	}

	/**
	 * Update taxes in the checkout processing process
	 */
	public function update_taxes_on_check_process() {
		
		if (wp_verify_nonce( isset($_REQUEST['_wpnonce']), 'my-nonce' ) ) {
			return false;
		}

		if (isset( $_POST['payment_method'] ) && 'cod' == sanitize_text_field( $_POST['payment_method'])) {
			return true;
		}
		
		global $woocommerce;
		$products = $woocommerce->cart->get_cart();
		$is_virtual = array();
		$has_physical = 'false';

		foreach ( $products as $product ) {

			$product_id = $product['product_id'];
			$is_virtual[] = get_post_meta( $product_id, '_virtual', true );	
		}
		if (in_array('no', $is_virtual)) {
			$has_physical = 'true';
		}

		if ('true' != $has_physical) {
			
			WC()->session->__unset( 'ship_product_class' );
			WC()->session->__unset( 'vertex_api_call' );
			return;

		} else {

			if (isset( $_POST['ship_to_different_address'] ) && '1' == sanitize_text_field( $_POST['ship_to_different_address'])) {
				if (isset( $_POST['shipping_country'] )) {
					$billing_country = sanitize_text_field( $_POST['shipping_country'] );
				}
			} else {
				if (isset( $_POST['billing_country'] )) {
					$billing_country = sanitize_text_field( $_POST['billing_country'] );
				}
			}

			// Transaction extra's
			$transaction_extra = array();

			if ( 'yes' == WC_TA_Integration::$enable_v1 || 'yes' == WC_TA_Integration::$enable_v2) {

				// Set force country code
				$transaction_extra['force_country_code'] = $billing_country;
			}

			// Check for VAT number
			if ( isset( $_POST['vat_number'] ) && '' !== $_POST['vat_number'] ) {
				$vat_number = sanitize_text_field( wp_unslash($_POST['vat_number']));
				// Set VAT number
				$transaction_extra['buyer_tax_number'] = str_replace( ' ', '', $vat_number );
			}


			// The cart manager
			$cart_manager = new WC_TA_Cart_Manager();

			// Setup request
			$request_calculate_tax = new WC_TA_Request_Calculate_Tax( $billing_country, $cart_manager->get_items_from_cart(), $transaction_extra );

			// Do request
			if ( $request_calculate_tax->do_request() ) {

				// Get the body
				$response_body = $request_calculate_tax->get_response_body();
				$rate = '';
				$rates = '';

				if ( isset( $response_body->transaction ) ) {
					$this->custom_shipping_rate($rates, $rate );
					// Update the taxes in cart based on transaction lines
					$this->update_taxes_in_cart( $response_body->transaction, $response_body->transaction->tax_country_codes );
				}
			} else {

				/**
				 * We're in the middle of processing the order so adding a notice should abort the process by validation
				 *
				 * @todo Better error handling.
				 */
				WC()->session->__unset( 'ship_product_class' );
				wc_add_notice( $request_calculate_tax->get_error_message(), 'error' );

			}
		}
	}

	/**
	 * Update the taxes when the line taxes are calculated in the admin
	 *
	 * @param array $items
	 * @param int $order_id
	 * @param String $country
	 *
	 * @return array
	 */
	public function update_taxes_on_calc_line_taxes( $items, $order_id, $country ) {
		$formatted_items = array();

		// Check for items
		if ( isset( $items['order_item_id'] ) ) {

			// Get the order
			$order = wc_get_order( $order_id );

			// Get the real items
			$full_items = $order->get_items( 'line_item' );

			// Loop through items
			foreach ( $full_items as $item_id => $item ) {

				// Get the product ID
				$product_id = wc_get_order_item_meta( $item_id, '_product_id', true );

				// Format the item so it can be turned into a transaction line
				$item['id']       = $product_id;
				$item['quantity'] = $item['qty'];
				$item['data']     = $order->get_product_from_item( $item );

				$formatted_items[ $item_id ] = $item;
			}

			// Setup request
			$request_calculate_tax = new WC_TA_Request_Calculate_Tax( $country, $formatted_items, array( 'force_country_code' => $country ) );

			// Do request
			if ( $request_calculate_tax->do_request() ) {

				// Get the body
				$response_body = $request_calculate_tax->get_response_body();

				if ( isset( $response_body->transaction ) ) {

					// Check
					if ( count( $response_body->transaction->transaction_lines ) > 0 ) {

						// Create tax manager object
						$tax_manager = new WC_TA_Tax_Manager();

						// Loop
						foreach ( $response_body->transaction->transaction_lines as $tl ) {

							// Don't modify tax rates for default products
							if ( 'default' == $tl->product_type ) {
								continue;
							}

							// Don't modify if tax isn't supported
							if ( isset( $response_body->transaction->tax_supported ) && false === $response_body->transaction->tax_supported ) {
								continue;
							}

							// Add the new product class for this product
							$tax_manager->add_product_tax_class( $tl->custom_id, $tl->product_type );

							// Add the new tax rate for this transaction line
							$tax_manager->add_tax_rate( $tl->product_type, $tl->tax_rate, strtoupper( 'VAT ' . $country . ' ' . $tl->product_type ) );

							// Add tax class to items
							$items['order_item_tax_class'][ $tl->custom_id ] = $tax_manager->clean_tax_class( $tl->product_type );
						}
					}
				}
			} else {

				/**
				 * We're in the middle of processing the order so adding a notice should abort the process by validation.
				 *
				 * @todo Better error handling.
				 */

				wc_add_notice( $request_calculate_tax->get_error_message(), 'error' );
			} // End if().
		} // End if().

		return $items;

	}

}
