<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
/**
 * WC_TA_Integration Class
 */
class WC_TA_Integration extends WC_Integration {

	public static $private_token = null;
	public static $public_token = null;
	public static $seller_token = null;
	public static $enable_v1 = null;
	public static $enable_v2 = null;
	public static $enable_v3 = null;
	public static $enable_mode = null;
	//public static $eu_country = null;
	public static $enable_vat_number = null;
	public static $show_taxamo_invoice = null;
	public static $universal_pricing = null;
	public static $taxamo_debug = null;

	/**
	 * Constructor
	 */
	public function __construct() {
		add_action( 'init', array( $this, 'setup_wizard_settings_option' ) );

		$this->id                 = 'taxamo';
		$this->method_title       = __( 'Taxamo Assure', 'woocommerce-taxamo' );
		
		if ( empty( self::$enable_mode) && 'production' != self::$enable_mode) {
			/* translators: %s: WooCommerce_Taxamo::TAXAMO_SANDBX_URL */
			$this->method_description = __( sprintf( 'Taxamo handles VAT calculation and reporting on physical goods &amp; services. %sYou need a Taxamo account for this extension to work, click here to sign up.%s', '<br/><a href="' . WooCommerce_Taxamo::TAXAMO_SANDBX_URL . '" target="_blank">', '</a>' ), 'woocommerce-taxamo' );
			
		} else {
			/* translators: %s: WooCommerce_Taxamo::TAXAMO_PROD_URL */
			$this->method_description = __( sprintf( 'Taxamo handles VAT calculation and reporting on physical goods &amp; services. %sYou need a Taxamo account for this extension to work, click here to sign up.%s', '<br/><a href="' . WooCommerce_Taxamo::TAXAMO_PROD_URL . '" target="_blank">', '</a>' ), 'woocommerce-taxamo' );
		}

		// Load admin form
		$this->init_form_fields();

		// Load settings
		$this->init_settings();
		self::$enable_v1 = 'no';
		self::$enable_v2 = 'no';
		self::$enable_v3 = $this->get_option( 'enable_v3', 'yes' );
		self::$private_token = $this->get_option( 'private_token' );
		//self::$eu_country = $this->get_option( 'eu_country' );
		self::$public_token  = $this->get_option( 'public_token' );
		self::$seller_token = $this->get_option( 'seller_token' );
		self::$taxamo_debug = $this->get_option( 'taxamo_debug', 'no' );
		self::$enable_vat_number       = $this->get_option( 'enable_vat_number', 'yes' );
		self::$show_taxamo_invoice	   = $this->get_option( 'show_taxamo_invoice', 'no' );
		self::$universal_pricing	   = $this->get_option( 'universal_pricing', 'no' );
		self::$enable_mode	   		   = $this->get_option( 'enable_mode');


		// Update option Hooks
		add_action( 'woocommerce_update_options_integration_taxamo', array( $this, 'process_admin_options' ) );

		// Filters.
		add_filter( 'woocommerce_settings_api_sanitized_fields_' . $this->id, array( $this, 'sanitize_settings' ) );

		if ( version_compare( WC_VERSION, '2.4.7', '>=' ) && 'yes' == self::$universal_pricing ) {
			add_filter( 'woocommerce_adjust_non_base_location_prices', '__return_false' );
		}

		if ( empty( self::$seller_token ) && empty( self::$private_token ) && empty( self::$public_token ) ) {
			add_action( 'admin_notices', array( $this, 'settings_notice' ) );
		}
	}

	/**
	 * Return Taxamo Universal Pricing settings for form fields
	 */
	public function get_universal_pricing_setting() {
	
		$universal_pricing_available = false;
		
		$woocommerce_prices_include_tax = ( get_option( 'woocommerce_prices_include_tax' ) == 'yes' ? true : false );
		$woocommerce_tax_display_shop   = ( get_option( 'woocommerce_tax_display_shop' ) == 'incl'  ? true : false );
		$woocommerce_tax_display_cart   = ( get_option( 'woocommerce_tax_display_cart' ) == 'incl'  ? true : false );
		
		// Check that conditions are met for this option to be enabled
		if ( $woocommerce_prices_include_tax && $woocommerce_tax_display_shop && $woocommerce_tax_display_cart ) {
			$universal_pricing_available = true;
		}
		
		if ( $universal_pricing_available ) {
			$setting = array(
				'title'       => __( 'Force Universal Pricing', 'woocommerce-taxamo' ),
				'description' => __( 'Enabling this option makes Taxamo calculate VAT in such a way, that the final price is always the same as the price provided.' ),
				'type'        => 'checkbox',
				'default'     => 'no',
				'desc_tip'    => __( 'Tick to enable consistent final pricing to customers in various juristictions.', 'woocommerce-taxamo' ),
				'disabled'	  => false,
				'tab'     => 'tab2',
			);
		} else {
			
			/* translators: %s: Build the description of what conditions must be met for this option to be enabled */
			$setting_description = __( sprintf( 'Customers will see the same product price across different jurisdictions, regardless of varying tax rates. Merchants absorb any differences in tax, ensuring a consistent pricing experience for customers.  Please refer to the <a href="https://docs.assure.taxamo.com/docs/why-use-taxamo-assure-physical#enable-universal-pricing-for-multiple-eu-member-states" target="blank">universal pricing docs</a> for further information.<br>In order for this option to be available you must set the following options on the %sTax Options%s page.', '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=tax' ) . '">', '</a>' ) );
			
			$setting_description .= '<ol>';
			
			$check = '';
			if ( $woocommerce_prices_include_tax ) {
				$check = 'wc-taxamo-setting-check';
			}
			/* translators: %s: setting_description */		
			$setting_description .= '<li class="' . $check . '"><span>' . __( sprintf( 'Prices Entered With Tax: %sYes, I will enter prices inclusive of tax%s', '<strong>', '</strong>' ) ) . '</span> </li>';

			$check = '';
			if ( $woocommerce_tax_display_shop ) {
				$check = 'wc-taxamo-setting-check';
			}

			/* translators: %s: setting_description */				
			$setting_description .= '<li class="' . $check . '"><span>' . __( sprintf( 'Display Prices in the Shop: %sIncluding tax%s', '<strong>', '</strong>' ) ) . '</span> </li>';

			$check = '';
			if ( $woocommerce_tax_display_cart ) {
				$check = 'wc-taxamo-setting-check';
			}

			/* translators: %s: setting_description */			
			$setting_description .= '<li class="' . $check . '"><span>' . __( sprintf( 'Display Prices During Cart and Checkout: %sIncluding tax%s', '<strong>', '</strong>' ) ) . '</span> </li>';
			
			$setting_description .= '</ol>';

			$setting = array( 
				'title'       => __( 'Force Universal Pricing', 'woocommerce-taxamo' ),
				'description' => $setting_description,
				'type'        => 'checkbox',
				'default'     => 'no',
				'disabled'	  => true,
				'tab'     => 'tab2',
			);
		}
		
		return $setting;
	}


	public function generate_custom_settings_html( $form_fields, $echo = true ) {
		// echo "<pre>";
		// print_r($form_fields);
		if ( empty( $form_fields ) ) {
			$form_fields = $this->get_form_fields();
		}

		$fields_tabs       = array();
		$second_tab_fields = array();
		foreach ( $form_fields as $key => $form_field ) {
			$fields_tabs[ $form_field['tab'] ][ $key ] = $form_field;
		}
		extract( $fields_tabs );

		include_once WC_TAXAMO_INTEGRATION_PLUGINDIR . '/includes/requests/template-ta-admin-settings.php';
	}

	public function admin_options() {
		$this->generate_custom_settings_html( $this->get_form_fields(), false ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}

	public function process_admin_options() {

		$seller_token = '';
		$enable_mode = '';

		if ( wp_verify_nonce( isset($_REQUEST['_wpnonce']), 'my-nonce' ) ) {
			return false;
		}

		if (isset($_POST['woocommerce_taxamo_seller_token'])) {
			$seller_token = trim(sanitize_text_field($_POST['woocommerce_taxamo_seller_token']));
		}

		if (isset($_POST['woocommerce_taxamo_enable_mode'])) {
			$enable_mode = trim(sanitize_text_field($_POST['woocommerce_taxamo_enable_mode']));
		}

		if ('production' === $enable_mode) {

			$URL = 'https://seller-transaction-api.marketplace.taxamo.com/api/v3/seller/tax/calculate';
		
		} else {

			$URL = 'https://seller-transaction-api.sandbox.marketplace.taxamo.com/api/v3/seller/tax/calculate';
		}
		$args['headers'] = array(
			'x-marketplace-seller-token' => $seller_token,
			
			'content-type'  => 'application/json',
		);
		$url = $URL;
		$response = wp_remote_post( $url, $args );
		
		
		if ('401' == $response['response']['code']) {
				update_option('tax_error', 'true');

			} else {
				delete_option('tax_error');
			}

		$get = get_option('tax_error');

		if (!empty($get)) {
			
				WC_Admin_Settings::add_error('Error: Please enter the valid Seller Token.');
				return false;
					
		}

		$saved = parent::process_admin_options();
		$this->init_form_fields();
		return $saved;
	}

	/**
	 * Init integration form fields
	 */
	public function init_form_fields() {

		$debug_label       = __( 'Enable Logging', 'woocommerce-taxamo' );
		$debug_description = __( 'Log Taxamo Assure Plugin events.  This is useful for Developers and the Taxamo Helpdesk to provide support and troubleshooting.  You can view the logs within WooCommerce at: Dashboard > WooCommerce > Status > <a href="' . esc_url( admin_url( 'admin.php?page=wc-status&tab=logs' ) ) . '" target="blank">Log</a>', 'woocommerce-taxamo' );

		if ( defined( 'WC_VERSION' ) && version_compare( WC_VERSION, '2.2', '>=' ) ) {
			$debug_label = sprintf( $debug_label, ' | <a href="' . esc_url( admin_url( 'admin.php?page=wc-status&tab=logs&log_file=' . esc_attr( $this->id ) . '-' . sanitize_file_name( wp_hash( $this->id ) ) . '.log' ) ) . '">' . __( 'View Log', 'woocommerce-taxamo' ) . '</a>' );
		} else {
			$debug_label = sprintf( $debug_label, ' | ' . __( 'View Log', 'woocommerce-taxamo' ) . ': <code>woocommerce/logs/' . $this->id . '-' . sanitize_file_name( wp_hash( $this->id ) ) . '.txt</code>' );
		}
		
		$this->form_fields = array(

			/*'enable_v1' => array(
				'title'       => __( 'Enable Version One', 'woocommerce-taxamo' ),
				'description' => __( 'Allow customers to version One.', 'woocommerce-taxamo' ),
				'type'        => 'checkbox',
				'class'		  => 'subject-list1',
				'default'     => 'yes'
			),
			'enable_v2' => array(
				'title'       => __( 'Enable Version Two', 'woocommerce-taxamo' ),
				'description' => __( 'Allow customers to version Two.', 'woocommerce-taxamo' ),
				'type'        => 'checkbox',
				'class'		  => 'subject-list2',
				'default'     => 'no'
			), 
			'enable_v3' => array(
				'title'       => __( 'Enable Version Three', 'woocommerce-taxamo' ),
				'description' => __( 'Allow customers to version Three (i.e, Taxamo Assure API).', 'woocommerce-taxamo' ),
				'type'        => 'checkbox',
				'class'		  => 'subject-list3',
				'default'     => 'yes',
				'tab'     => 'tab3'
			), */

			'enable_mode' => array(
				'title'		  => __( 'Environment', 'woocommerce-taxamo' ),
				'type'		  => 'select',
				'options'	  => array(
					'production'   => __( 'Production', 'woocommerce-taxamo' ),
					'sandbox'	   => __( 'Sandbox', 'woocommerce-taxamo' ),
				),
				'description'     	  => __( 'Use Sandbox for testing the service and Production for the live service.' ),
				'default' => 'sandbox',
				'tab'     => 'tab1',
			),
			'seller_token' => array(
				'title'		  => __( 'Seller Token', 'woocommerce-taxamo' ),
				'description' => __( sprintf( 'You can obtain your Seller token from your Taxamo account.  Please login (<a href="https://manage.sandbox.marketplace.taxamo.com/auth.html" target="blank">Sandbox</a> or <a href="https://manage.marketplace.taxamo.com/auth.html" target="blank">Production</a>) then navigate: Dashboard > Settings > View API tokens.'), 'woocommerce-taxamo' ),
				'type'        => 'text',
				'class'		  => 'subject-list',
				'tab'     	  => 'tab1',
			),

			/*'eu_country' => array(
				'title'		  => __( 'Choose Countries', 'woocommerce-taxamo' ),
				'name'        => __( 'Europian Country List', 'woocommerce-taxamo' ),
				'type'        => 'multiselect',
				'description' => __( 'Enable Taxamo service for selected countries.' ),
				'class'       => 'chzn-drop',
				'options'     => $this->set_EU_countries(),
				'default'     => array_keys($this->set_EU_countries()),
				'css'		  => 'height: 200px;'
			),
			'private_token' => array(
				'title'       => __( 'Private Token', 'woocommerce-taxamo' ),
				'description' => __( 'Get this token from your Taxamo account.', 'woocommerce-taxamo' ),
				'type'        => 'text',
				'class'		  => 'subject-list'
			),
			'public_token' => array(
				'title'       => __( 'Public Token', 'woocommerce-taxamo' ),
				'description' => __( 'Get this token from your Taxamo account.', 'woocommerce-taxamo' ),
				'type'        => 'text',
				'class'		  => 'subject-list'
			),*/
			'enable_vat_number' => array(
				'title'       => __( 'Enable VAT Number Field', 'woocommerce-taxamo' ),
				'description' => __( 'Allow businesses to enter their VAT number during checkout. Validated business transactions will be VAT exempt.', 'woocommerce-taxamo' ),
				'desc_tip'    => __( 'Tick to allow businesses to enter their VAT number during checkout.', 'woocommerce-taxamo' ),
				'type'        => 'checkbox',
				'default'     => 'yes',
				'tab'     => 'tab2',
			),
			'show_taxamo_invoice' => array(
				'title'       => __( 'Provide Invoice Link', 'woocommerce-taxamo' ),
				/* translators: %s: Show Invoices */
				'description' => __( sprintf( 'Taxamo Assure will send the customer the sole VAT invoice for their Assure transactions via email. Additionally, a link can be offered to the customer to see their invoice after checkout, if selected here. As an optional step: you can personalise the invoice template in your Taxamo Assure account. Please login (<a href="https://manage.sandbox.marketplace.taxamo.com/auth.html" target="blank">Sandbox</a> or <a href="https://manage.marketplace.taxamo.com/auth.html" target="blank">Production</a>) then navigate: Dashboard > Settings > Customize invoices.' ), 'woocommerce-taxamo' ),
				'desc_tip'    => __( 'Tick to provide customer with a link to their Taxamo Assure invoice after checkout.', 'woocommerce-taxamo' ),
				'type'        => 'checkbox',
				'default'     => 'no',
				'tab'     => 'tab2',
			),
			'taxamo_debug'   => array(
				'title'       => __( 'Debug Log', 'wc-oddo-integration' ),
				'type'        => 'checkbox',
				'label'       => $debug_label,
				'default'     => 'no',
				'description' => $debug_description,
				'desc_tip'    => __( 'Tick to enable logging Taxamo Assure Plugin events.', 'woocommerce-taxamo' ),
				'tab'     => 'tab2',
			),
		);
		
		if ( version_compare( WC_VERSION, '2.4.7', '>=' ) ) {
			// Get the universal pricing option and add it to the form fields array
			$this->form_fields[ 'universal_pricing' ] = $this->get_universal_pricing_setting();
		}
		
	}

	public function is_current_url() {

		// return $current_url;
		if ( isset( $_GET['tab'] ) && 'integration' == $_GET['tab'] && isset( $_GET['section'] ) && $this->id == $_GET['section'] ) {
			return true;
		}
		return false;
	}

	public function allowed_html() {
		$fields             = wp_kses_allowed_html( 'post' );
		$allowed_atts       = array(
			'align'      => array(),
			'class'      => array(),
			'type'       => array(),
			'id'         => array(),
			'dir'        => array(),
			'lang'       => array(),
			'style'      => array(),
			'xml:lang'   => array(),
			'src'        => array(),
			'alt'        => array(),
			'href'       => array(),
			'rel'        => array(),
			'rev'        => array(),
			'target'     => array(),
			'novalidate' => array(),
			'value'      => array(),
			'name'       => array(),
			'tabindex'   => array(),
			'action'     => array(),
			'method'     => array(),
			'for'        => array(),
			'width'      => array(),
			'height'     => array(),
			'data'       => array(),
			'title'      => array(),
			'label'      => array(),
			'checked'    => true,
			'select'     => array(),
			'option'     => array(),
			'selected'   => array(),
			'multiple'   => array(),
		);
		$fields['form']     = array(
			'action'         => true,
			'accept'         => true,
			'accept-charset' => true,
			'enctype'        => true,
			'method'         => array(),
			'name'           => true,
			'target'         => true,
			'class'          => true,
		);
		$fields['input']    = $allowed_atts;
		$fields['select']   = $allowed_atts;
		$fields['option']   = $allowed_atts;
		$fields['optgroup'] = $allowed_atts;

		$fields['script'] = $allowed_atts;
		$fields['style']  = $allowed_atts;

		return $fields;
	}

	/**
	 * Sanitize our settings
	 *
	 * @see process_admin_options()
	 */
	public function sanitize_settings( $settings ) {


		if ( isset( $settings['woocommmerce_taxamo'] ) ) {

			$settings = $settings;
		}

		return $settings;
	}

	/**
	 * Generate a URL to our TAP settings screen.
	 *
	 * @since  1.0.2
	 * @return string Generated URL.
	 */
	public function get_settings_url() {
		return add_query_arg(
			array(
				'page'    => 'wc-settings',
				'tab'     => 'integration',
				'section' => $this->id,
			),
			admin_url( 'admin.php' )
		);
	}


	/**
	 * Settings prompt
	 */
	public function settings_notice() {
		if ( ! empty( $_GET['tab'] ) && 'integration' === $_GET['tab'] ) {
			return;
		}
		?>
		<div id="message" class="updated woocommerce-message">
			<p><?php esc_attr_e( 'The Taxamo Assure Plugin is almost ready &#8211; Please add your Seller Token and complete the configuration to begin fetching tax rates.', 'woocommerce-taxamo' ); ?></p>

			<p class="submit"><a
					href="<?php echo esc_url(admin_url( 'admin.php?page=wc-settings&tab=integration&section=taxamo' )); ?>"
					class="button-primary"><?php esc_attr_e( 'Taxamo Assure Plugin Settings', 'woocommerce-taxamo' ); ?></a></p>
		</div>
		
	<?php
	}

	/**
	 * Settings prompt wizard 
	 */
	public function setup_wizard_settings_option() {
			


		if ( ! empty( $_GET['tab'] ) && 'integration' === $_GET['tab'] &&  ! empty( $_GET['tab'] ) && isset($_GET['section']) && 'taxamo' === $_GET['section']) {

			$wizard = get_option('woocommerce_taxamo_wizard_set', true);

			if (isset($wizard) && 'yes' != $wizard && empty(get_option('woocommerce_taxamo_settings'))) {	
				include( plugin_dir_path(( __FILE__ )) . '/class-wc-ta-wizard-template.php' );	
			}
		}
	}
}
