<?php

class WC_TA_Transaction_Manager {

	/**
	 * Build the transaction lines
	 *
	 * @param array $items
	 * @param String $country
	 *
	 * @return array
	 */
	private function build_transaction_lines( $items, $country ) {

		// Transaction lines
		$transaction_lines = array();

		if ('yes' === WC_TA_Integration::$enable_v3) {	// Lo0p
			
			if ( count( $items ) > 0 ) {

				// Country manager
				$country_manager = new WC_TA_Country_Manager();

				foreach ( $items as $item_key => $item ) {

					if ( isset( $item['data'] ) ) {

						// The transaction line
						$transaction_line = array();

						$product = $item['data'];
						if ('1' != $product->is_virtual()) {

						
							// Set the product Class Type
							$type = 'P';

							// Set the product title as description.
							$transaction_line['description'] = $product->get_title();

							// Set the product type
							$transaction_line['product_class'] = $type;

							// Custom ID
							$transaction_line['custom_id'] = '' . $item_key;

							// Quantity
							$transaction_line['quantity'] = $item['quantity'];

							// Price
							$woo_include_tax_option = get_option( 'woocommerce_prices_include_tax' );
							
							if ('yes' == $woo_include_tax_option ) {
									$transaction_line['total_amount'] = $product->get_price()*$item['quantity'];
									
							} else {

								$transaction_line['amount'] = $product->get_price()*$item['quantity'];
							}
							

							// What do we want? Floats! When do we want them? Now!

							// Check if amount is set
							if ( isset( $transaction_line['amount'] ) ) {
								$transaction_line['amount'] = floatval( $transaction_line['amount'] );
							}

							// Check if total_amount is set
							if ( isset( $transaction_line['total_amount'] ) ) {
								$transaction_line['total_amount'] = floatval( $transaction_line['total_amount'] );
							}

							// Get store country name
							$store_country = WC()->countries->countries[WC()->countries->get_base_country()];
							$store_address     = get_option( 'woocommerce_store_address' );
							$store_address_2   = get_option( 'woocommerce_store_address_2' );
							$store_city        = get_option( 'woocommerce_store_city' );
							$store_postcode    = get_option( 'woocommerce_store_postcode' );

							$WC_Countries = new WC_Countries();
							$ccodes = $WC_Countries->get_countries();
							$country_code = array_search ($store_country, $ccodes);
							$transaction_line['ship_from_address']	=  array(
											'street_name' => $store_address,
											'address_detail' => $store_address_2,
											'city' => $store_city,
											'postal_code' => $store_postcode,
											'country_code' => $country_code
										);

							$taxamo_product_cn_code = get_post_meta( $item['id'], 'taxamo_product_cn_code', true );
							if (!empty($taxamo_product_cn_code)) {
								
								$transaction_line['product_cn_code'] = $taxamo_product_cn_code;
							}
							
							// Float the tax rate
							//$transaction_line['tax_rate'] = floatval( $transaction_line['tax_rate'] );

							// Add transaction line to transaction lines
							$transaction_lines[] = $transaction_line;
						}	
					}
				}
			}
			
			// Get store country name
			$store_country = WC()->countries->countries[WC()->countries->get_base_country()];
			$store_address     = get_option( 'woocommerce_store_address' );
			$store_address_2   = get_option( 'woocommerce_store_address_2' );
			$store_city        = get_option( 'woocommerce_store_city' );
			$store_postcode    = get_option( 'woocommerce_store_postcode' );

			$WC_Countries = new WC_Countries();
			$ccodes = $WC_Countries->get_countries();
			$country_code = array_search ($store_country, $ccodes);

			if (isset(WC()->session->get('shipping_for_package_0')['rates'])) {
				
				foreach ( WC()->session->get('shipping_for_package_0')['rates'] as $method_id => $rate ) {
					if ( WC()->session->get('chosen_shipping_methods')[0] == $method_id ) {
						$rate_label = $rate->label; // The shipping method label name
						$vertex_ses = WC()->session->get( 'vertex_ses' );
						$old_shipping_amount = WC()->session->get( 'old_shipping_amount' );
						
						$rate_cost_tax = floatval($rate->cost);

						//$rate_cost_tax = $rate_cost_tax - $old_shipping_amount;

						$transaction_extra = array(
							'product_class' => 'S',
							'custom_id'    => '51315A00-DC44-408E-A575-8CE42A32AE1B',
							'quantity'     => 1,
							'amount' => wc_round_tax_total( $rate_cost_tax ),
							'description'  => $rate_label,
							'ship_from_address'	=>  array(
									'street_name' => $store_address,
									'address_detail' => $store_address_2,
									'city' => $store_city,
									'postal_code' => $store_postcode,
									'country_code' => $country_code
								),
									//'informative'  => true,
							);

						$transaction_lines[] = $transaction_extra;

					}
				}
			}

		} else { // For v1 and v2

			if ( count( $items ) > 0 ) {

				// Country manager
				$country_manager = new WC_TA_Country_Manager();

				foreach ( $items as $item_key => $item ) {
					$is_ebook = get_post_meta( $item['id'], '_ebook', true );

					$product = $item['data'];
					if ( isset( $item['data'] ) && '1' == $product->is_virtual() || 'yes' == $is_ebook) {

						// The transaction line
						$transaction_line = array();


						// Set the product type
						$type = 'e-service';

						$is_euexempt = get_post_meta( $item['id'], '_euexempt', true );

						// Check if this is a virtual product
						if ( $product->is_virtual() && 'yes' !== $is_euexempt ) {
							$type = 'e-service';
						} else {
							$transaction_line['informative'] = true;
						}

						// Check if this is an e-book
						

						if ( 'yes' === $is_ebook ) {
							$type = 'e-book';
						}

						// Check if Taxamo supports taxes for this country
						if ( false === $country_manager->is_tax_supported_for_country( $country ) ) {
							// Taxes aren't supported, set the line to informative
							$transaction_line['informative'] = true;
						}

						// Set the product title as description.
						$transaction_line['description'] = $product->get_title();

						// Set the product type
						$transaction_line['product_type'] = $type;

						// Custom ID
						$transaction_line['custom_id'] = '' . $item_key;

						// Quantity
						$transaction_line['quantity'] = $item['quantity'];

						// Price
						if ( $product->is_taxable() ) {

							// Always without tax
							$transaction_line['amount'] = $item['line_total'];

							// Get the base tax rates
							if ( method_exists( 'WC_Tax', 'get_base_tax_rates' ) ) {
								$base_tax_rates = WC_Tax::get_base_tax_rates( $product->get_tax_class() );
							} else {
								$base_tax_rates = WC_Tax::get_shop_base_rate( $product->get_tax_class() );
							}

							$base_tax_rate = array_shift( $base_tax_rates );
							
							if (!empty($base_tax_rate['rate'])) {
								// Set the tax rate
								$transaction_line['tax_rate'] = $base_tax_rate['rate'];
							} else {
								// Set 0 tax rate
								$transaction_line['tax_rate'] = 0;
							}

						} else {
							// Set Price
							$transaction_line['amount'] = $product->get_price();

							// Set 0 tax rate
							$transaction_line['tax_rate'] = 0;

							// Force non taxable
							$transaction_line['informative'] = true;
						}

						// What do we want? Floats! When do we want them? Now!

						// Check if amount is set
						if ( isset( $transaction_line['amount'] ) ) {
							$transaction_line['amount'] = floatval( $transaction_line['amount'] );
						}

						// Check if total_amount is set
						if ( isset( $transaction_line['total_amount'] ) ) {
							$transaction_line['total_amount'] = floatval( $transaction_line['total_amount'] );
						}

						// Float the tax rate
						$transaction_line['tax_rate'] = floatval( $transaction_line['tax_rate'] );

						// Add transaction line to transaction lines
						$transaction_lines[] = $transaction_line;
					}
				}
			}
		}

		return $transaction_lines;
	}

	/**
	 * Get a transaction array from the current cart
	 *
	 * @param String $country
	 * @param array $items
	 * @param null|String $buyer_name
	 * @param null|String $buyer_email
	 * @param null|String $custom_transaction_id
	 * @param array $transaction_extra
	 *
	 * @return array
	 */
	public function build_transaction( $country, $items, $buyer_name = null, $buyer_email = null, $custom_transaction_id = null, $transaction_extra = array(), $invoice_address = array() ) {

		// Country manager
		$country_manager = new WC_TA_Country_Manager();

		if ('yes' === WC_TA_Integration::$enable_v3) {	
			// The transaction
			if (isset($invoice_address) && isset($invoice_address['invoice_address'])) {
					$transaction = array(
						'currency_code'     => get_woocommerce_currency(),
						'ship_to_address' 	=> array(
							'country_code'	=> $invoice_address['invoice_address']['country'],
							'street_name'	=> $invoice_address['invoice_address']['street_one'],
							'address_detail'	=> $invoice_address['invoice_address']['street_two'],
							'city'	=> $invoice_address['invoice_address']['city'],
							'postal_code'	=> $invoice_address['invoice_address']['postal_code'],
							
						)
					);
				
			} else {
				$transaction = array(
					'currency_code'     => get_woocommerce_currency(),
					'ship_to_address' 	=> array(
						'country_code'	=> $country
					)
				);
			}
		}

		// Add buyer name
		if ( null !== $buyer_name ) {
			$transaction['buyer_name'] = $buyer_name;
		}

		// Add buyer email
		if ( null !== $buyer_email ) {
			$transaction['buyer_email'] = $buyer_email;
		}

		// Add custom transaction ID
		if ( null !== $custom_transaction_id ) {
			$transaction['custom_id'] = '' . $custom_transaction_id;
		}

		// Get the transaction lines
		$transaction['transaction_lines'] = $this->build_transaction_lines( $items, $country );
		// Merge transaction extra values
		if ( is_array( $transaction_extra ) && count( $transaction_extra ) > 0 ) {
			$transaction = array_merge_recursive( $transaction, $transaction_extra );
		}

		return $transaction;
	}

}

