<?php

/**
 * Class WC_TA_view_order
 *
 * Adds invoice link to view order page if enabled
 */
class WC_TA_View_Order {

	/**
	 * Setup the class
	 */
	public function setup() {
		add_action( 'woocommerce_view_order', array( $this, 'show_invoice_link' ) );
		add_action( 'woocommerce_thankyou', array( $this, 'show_invoice_link' ), 10, 1 );
	}
	
	
	/**
	 * Function show_invoice_link 
	 * Works out if an invoice link should be shown, if so, shows it
	 *
	 * @param $order_id
	 */
	public function show_invoice_link( $order_id ) {
		
		if ('yes' === WC_TA_Integration::$enable_v3) {
			
			if ( 'yes' == WC_TA_Integration::$show_taxamo_invoice ) {
				// Check to see if invoicing is turned on

				$order = wc_get_order( $order_id );		

				// Get the transaction key in order to buidl the link
				$vertex_transaction_key = opmc_hpos_get_post_meta( $order_id, 'vertex_transaction_key', true );

				// Make sure this only happens for completed orders, as invoices are not available for other statuses
				if ( ( !empty($vertex_transaction_key) && 'processing' == $order->get_status() ) || ( !empty($vertex_transaction_key) && 'completed' == $order->get_status() ) ) {
					$invoice_url = opmc_hpos_get_post_meta($order_id, 'vertex_invoice_url', true);
		
					// Load link template
					wc_get_template( 'view-order.php', array(
							'invoice_url' => $invoice_url
						),
						'woocommerce-view-order',
						untrailingslashit( plugin_dir_path( WooCommerce_Taxamo::get_plugin_file() ) ) . '/templates/' );
				}
			}
		} else {

			if ( 'yes' == WC_TA_Integration::$show_taxamo_invoice ) {
				// Check to see if invoicing is turned on

				$order = wc_get_order( $order_id );		

				// Get the transaction key in order to buidl the link
				$vertex_transaction_key = opmc_hpos_get_post_meta( $order_id, 'vertex_transaction_key', true );

				// Make sure this only happens for completed orders, as invoices are not available for other statuses
				if ( ( 'completed' == $order->get_status() && strlen( $vertex_transaction_key ) > 0 ) || ( 'processing' == $order->get_status() && strlen( $vertex_transaction_key ) > 0 )) {
					$invoice_url = opmc_hpos_get_post_meta($order_id, 'vertex_invoice_url', true);
					//$invoice_url = esc_url( "https://dashboard.taxamo.com/api/v1/transactions/$vertex_transaction_key/invoice?public_token=" . WC_TA_Integration::$public_token );
		
					// Load link template
					wc_get_template( 'view-order.php', array(
							'invoice_url' => $invoice_url
						),
						'woocommerce-view-order',
						untrailingslashit( plugin_dir_path( WooCommerce_Taxamo::get_plugin_file() ) ) . '/templates/' );
				}
			}
		}
	}
}
