<?php

/**
 * Class WC_TA_API_Request
 * Base API request class. Can't be initiated, must be extended.
 */
abstract class WC_TA_Request {

	// API URL base
	
	//const URL = 'https://api.taxamo.com/api/v1/';
	//https://api.taxamo.com/api/v1/tax/calculate

	//const URL = 'https://services.taxamo.com/api/v2/';  


	/**
	 * HTTP method
	 * Default is GET, can be overwritten.
	 *
	 * @var string
	 */
	private $http_method = 'GET';

	/**
	 * The private token
	 * Automatically loaded from WC options.
	 *
	 * @var string
	 */
	private $private_token = '';

	/**
	 * The public token
	 * Automatically loaded from WC options.
	 *
	 * @var string
	 */
	private $public_token = '';

	/**
	 * The private token
	 * Automatically loaded from WC options.
	 *
	 * @var string
	 */
	private $seller_token = '';

	/**
	 * The private token
	 * Automatically loaded from WC options.
	 *
	 * @var bool
	 */
	private $enable_v1 = '';

	/**
	 * The public token
	 * Automatically loaded from WC options.
	 *
	 * @var bool
	 */
	private $enable_v2 = '';

	/**
	 * The public token
	 * Automatically loaded from WC options.
	 *
	 * @var bool
	 */
	private $enable_v3 = '';

	/**
	 * The public token
	 * Automatically loaded from WC options.
	 *
	 * @var bool
	 */
	private $enable_mode = '';


	/**
	 * The request content type
	 * Default is JSON, can be overwritten.
	 *
	 * @var string
	 */
	private $content_type = 'application/json';

	/**
	 * The request user agent
	 * Set automatically by class.
	 *
	 * @var string
	 */
	private $user_agent = '';

	/**
	 * The request methods.
	 * Required
	 *
	 * @var array
	 */
	private $request_methods = null;

	/**
	 * The request endpoint
	 * Required
	 *
	 * @var string
	 */
	private $request_endpoint = null;

	/**
	 * The request body
	 * Optional
	 *
	 * @var array
	 */
	private $request_body = null;

	/**
	 * The request response
	 *
	 * @var array
	 */
	private $response = null;

	/**
	 * The error message
	 *
	 * @var string
	 */
	private $error_message = '';

	/**
	 * The debug log file
	 *
	 * @var string
	 */
	private $taxamo_debug = '';

	/* PLUGINS-2530 */
	/**
	 * The Universal Price
	 *
	 * @var string
	 */
	private $universal_pricing = '';
	
	/**
	 * The Show Taxamo Invoice
	 *
	 * @var string
	 */
	private $show_taxamo_invoice = '';
	/* PLUGINS-2530 END */
	

	/**
	 * The constructor
	 */
	public function __construct() {

		$this->enable_mode = WC_TA_Integration::$enable_mode;
		$this->enable_v2 = WC_TA_Integration::$enable_v2;
		$this->enable_v3 = WC_TA_Integration::$enable_v3;
		$this->private_token = WC_TA_Integration::$private_token;
		$this->public_token  = WC_TA_Integration::$public_token;
		$this->seller_token  = WC_TA_Integration::$seller_token;
		$this->user_agent    = 'Vertexwoo_v3_r1.0.2';
		$this->taxamo_debug = WC_TA_Integration::$taxamo_debug;
		$this->universal_pricing  = WC_TA_Integration::$universal_pricing;
		$this->show_taxamo_invoice  = WC_TA_Integration::$show_taxamo_invoice;

	}


	public function set_EU_countriesssss() {
		$countries = [
				'AT', 'BE', 'BG', 'CY',	'CZ', 'DE', 'DK', 'EE',	'ES', 'FI', 'FR', 'GR', 'HU', 'HR', 'IE', 'IT', 'LT', 'LU', 'LV', 'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK'
			];

		return $countries;
	}

	/**
	 * Build the request URL
	 *
	 * @return string
	 */
	private function build_request_url() {

		$checkver = WC_TA_Integration::$enable_v1; 
		$checkver2 = WC_TA_Integration::$enable_v2; 
		$checkver3 = WC_TA_Integration::$enable_v3; 

		if ('yes' == $checkver) {
			$URL = 'https://api.taxamo.com/api/v1/';
			 
		} else if ('yes' == $checkver2) {
			$URL = 'https://services.taxamo.com/api/v2/';
			 
		} else if ('yes' == $checkver3) {
			$enable_mode = $this->enable_mode;
			if ('production' === $enable_mode) {

				$URL = 'https://seller-transaction-api.marketplace.taxamo.com/api/v3/seller/';
			
			} else {

				$URL = 'https://seller-transaction-api.sandbox.marketplace.taxamo.com/api/v3/seller/';
			}		 	
			
		} else {

			return false;
		}

		// Build the URL
		$url = $URL;

		// Add request methods
		if ( ! is_null( $this->request_methods ) && is_array( $this->request_methods ) && count( $this->request_methods ) > 0 ) {
			$url .= implode( '/', $this->request_methods ) . '/';
		}

		// Add the endpoint
		if ( ! is_null( $this->request_endpoint ) && '' != $this->request_endpoint ) {
			$url .= $this->request_endpoint;
		}

		// Add the request body if this is a GET request
		if ( 'GET' == $this->http_method && ! is_null( $this->request_body ) && is_array( $this->request_body ) && count( $this->request_body ) > 0 ) {
			$url .= '?' . build_query( $this->request_body );
		}

		return $url;
	}

	/**
	 * Clear the request variable
	 *
	 * @return bool
	 */
	private function clear_response() {
		$this->response = null;

		return true;
	}

	/**
	 * Set the error message
	 *
	 * @param $error_message
	 *
	 * @return bool
	 */
	private function set_error_message( $error_message ) {
		$this->error_message = $error_message;

		return true;
	}

	/**
	 * Set the request method
	 *
	 * @param $request_methods
	 *
	 * @return bool
	 */
	protected function set_request_methods( $request_methods ) {
		$this->request_methods = $request_methods;

		return true;
	}

	/**
	 * Set the request endpoint
	 *
	 * @param $endpoint
	 *
	 * @return bool
	 */
	protected function set_request_endpoint( $endpoint ) {
		$this->request_endpoint = $endpoint;

		return true;
	}

	/**
	 * Set the request body
	 *
	 * @param array $request_body
	 *
	 * @return bool
	 */
	protected function set_request_body( $request_body ) {
		$this->request_body = $request_body;

		return true;
	}

	/**
	 * Set the HTTP method
	 *
	 * @param string $http_method
	 */
	protected function set_http_method( $http_method ) {
		$this->http_method = $http_method;
	}

	/**
	 * Get the error message
	 *
	 * @return string
	 */
	public function get_error_message() {
		return $this->error_message;
	}

	public function addLog( $message) {
		//$message = 'Unable To Update Record For ';
		if ('yes' == $this->taxamo_debug) {
			$wcLogger =  WooCommerce_Taxamo::get_logger();
			$wcLogger->add('Taxamo', $message);
		}
	}

	/**
	 * Do the request
	 *
	 * @return bool
	 */
	public function do_auth_request() {

		if ( !empty($this->request_body ) ) {

			$data = $this->request_body;
			$enable_v3 = $data['auth']['enable_v3'];
			$enable_v2 = $data['auth']['enable_v2'];
			$enable_v1 = $data['auth']['enable_v1'];
			$seller_token = $data['auth']['seller_token'];
			$private_token = $data['auth']['private_token'];
			$public_token = $data['auth']['public_token'];
			$enable_mode = $data['auth']['enable_mode'];
			
			if (!empty($enable_v1) && '1' == $enable_v1) {
				$URL = 'https://api.taxamo.com/api/v1/';
				 
			} else if (!empty($enable_v2) && '1' == $enable_v2 ) {
				$URL = 'https://services.taxamo.com/api/v2/';
				 
			} else if (!empty($enable_v3) && '1' == $enable_v3) {
				if ('production' === $enable_mode) {

					$URL = 'https://seller-transaction-api.marketplace.taxamo.com/api/v3/seller/';
				
				} else {

					$URL = 'https://seller-transaction-api.sandbox.marketplace.taxamo.com/api/v3/seller/';
				}		 	
				
			} else {

				return false;
			}

			// Build the URL
			$url = $URL;

			// Add request methods
			if ( ! is_null( $this->request_methods ) && is_array( $this->request_methods ) && count( $this->request_methods ) > 0 ) {
				$url .= implode( '/', $this->request_methods ) . '/';
			}

			// Add the endpoint
			if ( ! is_null( $this->request_endpoint ) && '' != $this->request_endpoint ) {
				$url .= $this->request_endpoint;
			}

			if (!empty($enable_v3) && '1' == $enable_v3) {

				$args['headers'] = array(
					'x-marketplace-seller-token' => $seller_token,
					'user-agent'	=> 'Vertexwoo_v3_r1.0.2',
					'content-type'  => $this->content_type,
					'source-id'     => get_site_url() . ' - woocommerce-taxamo - ' . WOOCOMMERCE_TAXAMO_VERSION,
				);

			} else {
			
				$args['headers'] = array(
					'Private-Token' => $private_token,
					'Public-Token'  => $public_token,
					'Content-Type'  => $this->content_type,
					'Source-Id'     => get_site_url() . ' - woocommerce-taxamo - ' . WOOCOMMERCE_TAXAMO_VERSION,
				);
			}
		}
		$msg = 'Initiating Auth request...' . PHP_EOL . print_r($args, true);
		$this->addLog($msg);

		$this->response = wp_remote_post( $url, $args );
		
		if ('1' == $enable_v3 ) {
			if ('401' == $this->response['response']['code']) {
				update_option('tax_error', 'true');
				$msg = $this->response['response']['code'] . ' - Unauthorized, please enter valid seller token!';
				$this->addLog($msg);
				return 'true';

			} else {
				delete_option('tax_error');
				$msg = $this->response['response']['code'] . ' - Access authorised successfully!';
				$this->addLog($msg);
				return 'false';
			}
			
		} elseif ( '1' == $enable_v2 ) {

			if ('400' == $this->response['response']['code'] && 'Invalid token.' == $this->response['body'] || 'Permission denied.' == $this->response['body']) {
				update_option('tax_error', 'true');
				return 'true';
			} else {
				delete_option('tax_error');
				return 'false';

			}
		} elseif ( '1' == $enable_v1 ) {

			if ('401' == $this->response['response']['code']) {

				update_option('tax_error', 'true');
				return 'true';
			} else {
				delete_option('tax_error');
				return 'false';

			}
		} else {
			
			return 'false';
		}	
		return 'false';
	}

	/**
	 * Do the request
	 *
	 * @return bool
	 */
	public function do_request() {

		// debug log related.
		$woo_tax_option = get_option( 'woocommerce_prices_include_tax' );
		$woo_tax_display_shop  = ( get_option( 'woocommerce_tax_display_shop' ) == 'incl'  ? 'Including tax' : 'Excluding tax' );
		$woo_tax_display_cart_checkout   = ( get_option( 'woocommerce_tax_display_cart' ) == 'incl'  ? 'Including tax' : 'Excluding tax' );
		
		if ( 'yes' == $woo_tax_option ) {
			$woo_tax_option = 'Yes, I will enter prices inclusive of tax';
		} else {
			$woo_tax_option = 'No, I will enter prices exclusive of tax';
		}

		if ('yes' == $this->taxamo_debug && 'calculate' == $this->request_endpoint) {
					
			$msg = 'Woo Setting: ' . PHP_EOL . 'Prices entered with tax => ' . $woo_tax_option . PHP_EOL . 'Taxamo Settings:' . PHP_EOL . 'Environment => ' . $this->enable_mode . ', Show Invoices => ' . $this->show_taxamo_invoice . ',  Force Universal Pricing => ' . $this->universal_pricing . PHP_EOL;

				$this->addLog($msg);
		}

		$checkver3 = $this->enable_v3; 
		$args = array(
				'method'     => $this->http_method,
				'timeout'    => 70,
				'sslverify'  => false,
				'user-agent' => 'Vertexwoo_v3_r1.0.2'
			);

		if ('yes' == $checkver3) {

			$avaiable_country = $this->set_EU_countriesssss();

			$b_country_code = array();

			if (!empty($this->request_body) && array_key_exists('transaction', $this->request_body)) {
				
				$get_add = $this->request_body;
				$b_country_code = $get_add['transaction']['ship_to_address']['country_code'];
			}

			if ('transactions' == $this->request_endpoint ) {

				$get_add = $this->request_body;
				$count_line_items = count($get_add['transaction']['transaction_lines']);
				$line_arr = $get_add['transaction']['transaction_lines'];
				foreach ($line_arr as $value) {
					$ship_amount = $value['amount'];
				}
				if (0 == $ship_amount) {
					unset($line_arr[$count_line_items-1]);
				} else {
					unset($line_arr[$count_line_items-2]);
				}
				
				$line_arr2 = array_values($line_arr);
				$this->request_body['transaction']['transaction_lines'] = $line_arr2;

			}

			if ('calculate' == $this->request_endpoint && array_key_exists('transaction', $this->request_body)) {

				if (!in_array($b_country_code, $avaiable_country)) {

				 //wc_add_notice(esc_attr('Selected country not authorised to process further Tax Calculation. Please connect with administrator!.', 'taxamo'), 'notice');
					return true;
				}

				global $woocommerce;
				$cartsubtotal = $woocommerce->cart->get_subtotal();
				if (isset($cartsubtotal) && '150' <= $cartsubtotal) {

					 //wc_add_notice(esc_attr('Cart Subtotal Amount Exceeded to the Taxamo Maximum Allowed TaxCalculate Amount.', 'taxamo'), 'notice');
					
					return true;
				}
			}

			$args['headers'] = array(
					'x-marketplace-seller-token' => $this->seller_token,
					'user-agent'	=> 'Vertexwoo_v3_r1.0.2',
					'content-type'  => $this->content_type,
					'source-id'     => get_site_url() . ' - woocommerce-taxamo - ' . WOOCOMMERCE_TAXAMO_VERSION,
			);

		} else {
			
			$args['headers'] = array(
					'Private-Token' => $this->private_token,
					'Public-Token'  => $this->public_token,
					'Content-Type'  => $this->content_type,
					'Source-Id'     => get_site_url() . ' - woocommerce-taxamo - ' . WOOCOMMERCE_TAXAMO_VERSION,
			);
		}

		//Add the request body if we've got one
		if ( ! is_null( $this->request_body ) && is_array( $this->request_body ) && count( $this->request_body ) > 0 ) {
			$args['body'] = json_encode( $this->request_body );
		}

		$request_body = json_encode($this->request_body, JSON_PRETTY_PRINT);
		$msg = 'Initiating ' . $this->request_endpoint . ' API request...' . PHP_EOL . $request_body . PHP_EOL;
		$this->addLog($msg);

		// Set Response
		$this->response = wp_remote_post( $this->build_request_url(), $args );

			
				
		if ( is_wp_error( $this->response ) ) {
			$this->set_error_message( __( 'There was a problem connecting to the API.', 'woocommerce-taxamo' ) );
			$this->addLog('There was a problem connecting to the API.');
			$this->clear_response();

			return false;
		}

		// For debug log
		if ( 'yes' == $this->taxamo_debug && isset( $this->response['response'] ) && '200' == $this->response['response']['code'] ) {
			$lineres = array();
			$response = json_decode($this->response['body'], true);
			unset($response['transaction']['required_fields']);
			unset($response['transaction']['invoice_timestamp']);
			unset($response['transaction']['countries']);
			unset($response['transaction']['discounts']);
			unset($response['transaction']['location_evidence']);
			unset($response['transaction']['vouchers']);
			unset($response['transaction']['tax_timestamp']);
			$transaction_lines = $response['transaction']['transaction_lines'];

			foreach ($transaction_lines as $value) {
				unset($value['reverse_charge']);
				unset($value['tax_exempt']);
				unset($value['additional_currencies']);
				unset($value['tax_number_service']);
				unset($value['tax_liability_owner_code']);
				unset($value['tax_supported']);
				unset($value['is_seller_permanent']);
				unset($value['line_num']);
				$lineres[] = $value;

			}

			unset($response['transaction']['transaction_lines']);
			$line_res['transaction_lines'] = $lineres;
			$final_response = array_merge( $response, $line_res );
			$json_response = json_encode($final_response, JSON_PRETTY_PRINT);
			
			$msg = 'Getting response from ' . $this->request_endpoint . ' API call.' . PHP_EOL . $json_response . PHP_EOL;
				$this->addLog($msg);
		}

		// Check response code
		if ( ! isset( $this->response['response'] ) || '200' != $this->response['response']['code'] ) {

			// Check if it's a transaction confirm request with a 400 response code
			if ( 'confirm' === $this->request_endpoint && 400 === $this->response['response']['code'] ) {
				$this->addLog('Transaction already confirmed.');

				// 400 at confirm means transaction already confirmed, we can ignore this error
				return true;

			}

			/*if ( 400 == $this->response['response']['code'] && 'yes' != $checkver3 ) {

				$error_message = apply_filters( 'woocommerce_taxamo_validation_error_message_location', __( 'We couldn\'t confirm your location.', 'woocommerce-taxamo' ) );

				if ( 'yes' === WC_TA_Integration::$enable_self_declaration ) {
					$error_message .= apply_filters( 'woocommerce_taxamo_validation_error_message_manually_validate', __( ' Please manually validate it.', 'woocommerce-taxamo' ) );
				}

				// Set error message
				$this->set_error_message( $error_message );

				// Allow user to self declare his location
				if ( isset( WC()->session ) ) {
					WC()->session->set( 'wc_ta_location_confirmation', true );
				}
			}*/

			// Clear response
			$this->clear_response();

			return false;
		}
		return true;
	}

	/**
	 * Get the response body
	 *
	 * @return array
	 */
	public function get_response_body() {
		if ( ! is_null( $this->response ) ) {
			return json_decode( $this->response['body'] );
		}

		return array();
	}

}
