<?php
/**
 * Plugin Name: WooCommerce Taxamo Assure
 * Plugin URI: https://woocommerce.opmc.com.au/
 * Description: Use Taxamo services in your WooCommerce shop.
 * Version: 1.1.0
 * Author: OPMC
 * Author URI: https://opmc.com.au/
 * Text Domain: woocommerce-taxamo-assure
 * Domain Path: /languages
 * License: GPL v3
 * Requires at least: 4.5
 * Tested up to: 6.4
 * WC requires at least: 3.0
 * WC tested up to: 8.7
 *
 * Woo:
 *
 * Copyright (c) 2017 WooCommerce
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Required functions
 */
if ( ! function_exists( 'woothemes_queue_update' ) ) {
	require_once( plugin_dir_path( __FILE__ ) . '/woo-includes/woo-functions.php' );
}
require_once 'opmc-hpos-compatibility-helper.php' ;
// Enable updates
//woothemes_queue_update( plugin_basename( __FILE__ ), 'f4234d8df5202ab6d8162c4c0e50cb3c', '583608' );

define( 'WOOCOMMERCE_TAXAMO_VERSION', '1.0' );
define( 'WC_TAXAMO_INTEGRATION_PLUGINDIR', plugin_dir_path( __FILE__ ) );

/* handle initial activation */
register_activation_hook( __FILE__, 'taxamo_check_wizard_exists' );

function taxamo_check_wizard_exists() {

	if (empty(get_option('woocommerce_taxamo_wizard_set')) && empty(get_option('woocommerce_taxamo_settings'))) {
		update_option('woocommerce_taxamo_wizard_set', 'no');
	}
	
}

function delete_plugin_database_table(){
    delete_option('woocommerce_taxamo_wizard_set');
    delete_option('woocommerce_taxamo_settings');
}

register_uninstall_hook(__FILE__, 'delete_plugin_database_table');


/**
 * Plugin page links
 */
function wc_tap_plugin_links( $links ) {

	$plugin_links = array(
		'<a href="' . admin_url( 'admin.php?page=wc-settings&tab=integration&section=taxamo' ) . '">' . __( 'Settings', 'woocommerce-taxamo' ) . '</a>',
	);

	return array_merge( $plugin_links, $links );
}

add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'wc_tap_plugin_links' );

/**
 * Class WooCommerce_Taxamo
 *
 * @since 1.0.0
 */
class WooCommerce_Taxamo {

	const TAXAMO_SANDBX_URL = 'https://manage.sandbox.marketplace.taxamo.com/auth.html#self-signup/Taxamo';
	const TAXAMO_PROD_URL = 'https://manage.marketplace.taxamo.com/auth.html#self-signup/Taxamo';
	const TAXAMO_URL = 'https://manage.marketplace.taxamo.com/auth.html#self-signup/Taxamo';
	

	/**
	 * Get the plugin file
	 *
	 * @static
	 * @since  1.0.0
	 *
	 * @return String
	 */
	public static function get_plugin_file() {
		return __FILE__;
	}

	/**
	 * A static method that will setup the autoloader
	 *
	 * @static
	 * @since  1.0.0
	 * 
	 */
	private static function setup_autoloader() {
		require_once( plugin_dir_path( self::get_plugin_file() ) . '/includes/class-wc-ta-autoloader.php' );
		$autoloader = new WC_TA_Autoloader();
		spl_autoload_register( array( $autoloader, 'load' ) );
	}

	/**
	 * Constructor
	 */
	public function __construct() {

		// Check if WC is activated
		if ( ! WC_Dependencies::woocommerce_active_check() ) {
			add_action( 'admin_notices', array( $this, 'notice_activate_wc' ) );
		} elseif ( version_compare( WC_VERSION, '2.2.9', '<' ) ) {
			add_action( 'admin_notices', array( $this, 'notice_version_wc' ) );
		} else {
			$this->init();
		}

		// Add Taxamo Admin CSS
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_css' ) );

		// Add Taxamo Admin JS
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_js' ) );

		$wizard = get_option('woocommerce_taxamo_wizard_set', true);

		if (isset($wizard) && 'yes' != $wizard && empty(get_option('woocommerce_taxamo_settings'))) {

			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_popup_js' ) );
		}
		
		add_action( 'wp_footer', array( $this, 'enqueue_checkout_js') );
		
		add_action( 'wp_ajax_get_tap_wizard_data', array( $this, 'get_tap_wizard_data' ) );
		add_action( 'wp_ajax_nopriv_get_tap_wizard_data', array( $this, 'get_tap_wizard_data' ) );
		add_action(	'before_woocommerce_init',
				function () {
				if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
					\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
				}
			}
		);
	}

	public function get_tap_wizard_data() {
		$environment = isset( $_REQUEST['environment'] ) ? sanitize_text_field( $_REQUEST['environment'] ) : '';
		$seller_token = isset( $_REQUEST['seller_token'] ) ? sanitize_text_field( $_REQUEST['seller_token'] ) : '';
		$vat_numer = isset( $_REQUEST['vat_numer'] ) ? sanitize_text_field( $_REQUEST['vat_numer'] ) : '';
		$invoice = isset( $_REQUEST['invoice'] ) ? sanitize_text_field( $_REQUEST['invoice'] ) : '';
		$universal_price = isset( $_REQUEST['universal_price'] ) ? sanitize_text_field( $_REQUEST['universal_price'] ) : '';

		$debug_log = isset( $_REQUEST['debug_log'] ) ? sanitize_text_field( $_REQUEST['debug_log'] ) : '';

		if (!empty($environment) && !empty($seller_token )) {

			add_option('woocommerce_taxamo_settings');
			$setting_data = array();

			update_option('woocommerce_taxamo_wizard_set', 'yes');

			$setting_data['enable_mode'] = $environment;
			$setting_data['seller_token'] = $seller_token;
			$setting_data['enable_vat_number'] = $vat_numer;
			$setting_data['show_taxamo_invoice'] = $invoice;
			$setting_data['taxamo_debug'] = $debug_log;
			$setting_data['universal_pricing'] = $universal_price;

			update_option('woocommerce_taxamo_settings', $setting_data);

		}

		echo 'success';
		wp_die();
	}




	public static function get_logger() {
		global $woocommerce;

		if ( class_exists( 'WC_Logger' ) ) {
			return new WC_Logger();
		} else {
			return $woocommerce->logger();
		}
	}

	/**
	 * Display the notice
	 *
	 * @since  1.0.0
	 *
	 */
	public function notice_activate_wc() {
		?>
		<div class="error">
			<p>
			<?php
			/* translators: 1: href link for the extension 2: closing href tag */
			printf( esc_html(__( 'Please install and activate %1$sWooCommerce%2$s in order for the WooCommerce Taxamo extension to work!', 'woocommerce-taxamo' )), '<a href="' . esc_url(admin_url( 'plugin-install.php?tab=search&s=WooCommerce&plugin-search-input=Search+Plugins' ) ) . '">', '</a>' ); 
			?>
			</p>
		</div>
	<?php
	}

	/**
	 * Display the notice
	 *
	 * @since  1.0.0
	 *
	 */
	public function notice_version_wc() {
		?>
		<div class="error">
			<p><?php esc_html_e( 'Please update WooCommerce to version 2.2.9 or higher in order for the WooCommerce Taxamo extension to work!', 'woocommerce-taxamo' ); ?></p>
		</div>
	<?php
	}

	/**
	 * Init the plugin
	 *
	 * @since  1.0.0
	 *
	 */
	private function init() {

		// Load plugin textdomain
		load_plugin_textdomain( 'woocommerce-taxamo', false, plugin_dir_path( self::get_plugin_file() ) . 'languages/' );

		// Setup the autoloader
		self::setup_autoloader();

		// The VAT number Field
		$vat_number_field = new WC_TA_Vat_Number_Field();
		$vat_number_field->setup();

		// Setup the Checkout VAT stuff
		$checkout_vat = new WC_TA_Checkout_Vat();
		$checkout_vat->setup();

		// Setup Taxamo manager
		$taxamo_manager = new WC_TA_Taxamo_Manager();
		$taxamo_manager->setup();

		// Setup View Order invoice link
		$view_order_invoice_link = new WC_TA_View_Order();
		$view_order_invoice_link->setup();

		// Admin only classes
		if ( is_admin() ) {

			// The admin E-Book Field
			//$admin_ebook = new WC_TA_Admin_Ebook();
			//$admin_ebook->setup();

			// The admin EU Exempt Field
			//$admin_euexempt = new WC_TA_Admin_EUexempt();
			//$admin_euexempt->setup();

			// The admin Taxamo CN code Field
			$admin_cncode = new WC_TA_Admin_CNcode();
			$admin_cncode->setup();

			// Setup the reports
			$reports = new WC_TA_Reports();
			$reports->setup();

			// Filter plugin links
			add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_links' ) );
		}

		// Add Taxamo integration fields
		add_filter( 'woocommerce_integrations', array( $this, 'load_integration' ) );

		// Enqueue scripts
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		// Subscriptions compatibility
		if ( class_exists( 'WC_Subscriptions' ) ) {
			$sc = new WC_TA_Subscriptions_Compatibility();
			$sc->setup_hooks();
		}

	}

	/**
	 * Plugin page links
	 */
	public function plugin_links( $links ) {

		$plugin_links = array(
			'<a href="' . self::TAXAMO_URL . '" target="_blank">' . __( 'Sign Up', 'woocommerce-taxamo' ) . '</a>',
		);

		return array_merge( $plugin_links, $links );
	}

	/**
	 * Define integration
	 *
	 * @param  array $integrations
	 *
	 * @return array
	 */
	public function load_integration( $integrations ) {
		$integrations[] = 'WC_TA_Integration';

		return $integrations;
	}

	/**
	 * Enqueue the VAT field scripts
	 */
	public function enqueue_scripts() {
		if ( is_checkout() ) {

			wp_enqueue_script(
				'wc_ta_checkout_js',
				plugins_url( '/assets/js/checkout' . ( ( ! SCRIPT_DEBUG ) ? '.min' : '' ) . '.js', self::get_plugin_file() ),
				array( 'jquery' ),
				'1.0'
			);

			$wc_countries = new WC_Countries();

			wp_localize_script( 'wc_ta_checkout_js', 'wc_taxamo', array(
				'eu_countries' => json_encode( $wc_countries->get_european_union_countries( 'eu_vat' ) ),
			) );

			wp_enqueue_style(
				'wc_af_post_shop_order_css',
				plugins_url( '/assets/css/woocommerce-taxamo.css', self::get_plugin_file() ),
				array(),
				'1.0'
			);

		}
	}

	/**
	 * Enqueue the Admin CSS scripts
	 */
	public function enqueue_admin_css() {
		wp_enqueue_style(
			'wc_taxamo_admin_css',
			plugins_url( '/assets/css/woocommerce-taxamo-admin.css', self::get_plugin_file() ),
			array(),
			'1.0'
		);
	}

	/**
	 * Enqueue the Admin JS scripts
	 */
	public function enqueue_admin_js() {
		wp_enqueue_script(
			'wc_taxamo_admin_js',
			plugins_url( '/assets/js/admin-order-view.js', self::get_plugin_file() ),
			array('jquery'),
			'1.0'
		);
	}

	/**
	 * Enqueue the Admin JS scripts
	 */
	public function enqueue_checkout_js() {
		wp_enqueue_script(
			'wc_taxamo_checkout_js',
			plugins_url( '/assets/js/checkout.js', self::get_plugin_file() ),
			array('jquery'),
			'1.0'
		);
	}

	public function enqueue_admin_popup_js() {


		if ( ! empty( $_GET['tab'] ) && 'integration' === $_GET['tab'] &&  ! empty( $_GET['tab'] ) && isset($_GET['section']) && 'taxamo' === $_GET['section']) {
			
			$wizard = get_option('woocommerce_taxamo_wizard_set');
			if (isset($wizard) && 'yes' != $wizard && empty(get_option('woocommerce_taxamo_settings'))) {

				wp_enqueue_script('taxamo-jquery.min.js', plugins_url( '/assets/js/jquery.min.js', self::get_plugin_file() ), array('jquery'), '3.2.1' );
				
				wp_enqueue_script('taxamo-bootstrap.min.js', plugins_url( '/assets/js/bootstrap.min.js', self::get_plugin_file() ), array('jquery'), '4.3.1' );

				
				wp_enqueue_script('wc_taxamo_popper_js', plugins_url( '/assets/js/popper.min.js', self::get_plugin_file() ), array('jquery'), '1.12.9' );

				wp_enqueue_script('wc_taxamo_jquery_smartWizard_js', plugins_url( '/assets/js/jquery.smartWizard.min.js', self::get_plugin_file() ), array('jquery'), 'v1581152197' );

				wp_enqueue_script('jquery-ui.min.js', 'https://ajax.googleapis.com/ajax/libs/jqueryui/1.10.3/jquery-ui.min.js', array('jquery'), '1.10.3' );


				wp_enqueue_style('wc_taxamo_bootstrap_css',	plugins_url( '/assets/css/bootstrap.min.css', self::get_plugin_file() ), array(), '4.3.1' );

				wp_enqueue_style('wc_taxamo_smart_wizard_css', plugins_url( '/assets/css/smart_wizard.min.css', self::get_plugin_file() ), array(), 'v1581152092' );
				
				wp_enqueue_style('taxamo-smart_wizard_theme_dots.min.css', plugins_url( '/assets/css/smart_wizard_theme_dots.min.css', self::get_plugin_file() ), array(), 'v1581152092' );
			}
		}
	}
}

// The 'main' function
function woocommerce_taxamo_main() {
	new WooCommerce_Taxamo();
}

// Create object - Plugin init
add_action( 'plugins_loaded', 'woocommerce_taxamo_main' );
